// BonDriverProxy.cpp: BonDriverNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "BonDriverProxy.h"


/////////////////////////////////////////////////////////////////////////////
// BonDriver - IHalTsTunerϊNX
/////////////////////////////////////////////////////////////////////////////

// IBonDriver
const BOOL CBonDriverProxy::OpenTuner(void)
{
	// `[iI[v
	const BOOL bReturn = (m_pTsTuner->OpenTuner())? TRUE : FALSE;

	// I[vtOZbg
	m_bIsOpen = bReturn;

	return bReturn;
}

void CBonDriverProxy::CloseTuner(void)
{
	// `[i
	m_pTsTuner->CloseTuner();
	
	// I[vtONA
	m_bIsOpen = FALSE;
}

const BOOL CBonDriverProxy::SetChannel(const BYTE bCh)
{
	// `[jOԂ̎w肪oȂߔΉƂ
	::BON_ASSERT(false, TEXT("IBonDriver::SetChannel()͎Ă܂"));
	
	return FALSE;
}

const float CBonDriverProxy::GetSignalLevel(void)
{
	// CNRԂ
	return m_pTsTuner->GetSignalLevel();
}

const DWORD CBonDriverProxy::WaitTsStream(const DWORD dwTimeOut)
{
	// Xg[EFCg
	switch(m_pTsTuner->WaitStream(dwTimeOut)){
		case CSmartEvent::WE_CLOSED :
			// nhꂽ
			return WAIT_ABANDONED;

		case CSmartEvent::WE_TRIGGER :
			// CxggK
			return (GetReadyCount())? WAIT_OBJECT_0 : WAIT_ABANDONED;
		
		case CSmartEvent::WE_TIMEOUT :
			// ^CAEg
			return WAIT_TIMEOUT;

		default:
			// O
			return WAIT_FAILED;
		}
}

const DWORD CBonDriverProxy::GetReadyCount(void)
{
	// o\Xg[Ԃ
	return m_pTsTuner->GetAvailableNum();
}

const BOOL CBonDriverProxy::GetTsStream(BYTE *pDst, DWORD *pdwSize, DWORD *pdwRemain)
{
	// Xg[擾
	BYTE *pData = NULL;
	DWORD dwSize = 0UL;
	DWORD dwRemain = 0UL;
	
	// Xg[|C^擾
	if(!m_pTsTuner->GetStream(&pData, &dwSize, &dwRemain))return FALSE;
	
	// Xg[Rs[
	if(pDst && pdwSize)::CopyMemory(pDst, pData, dwSize);
	if(pdwSize)*pdwSize = dwSize;
	if(pdwRemain)*pdwRemain = dwRemain;
	
	return TRUE;
}

const BOOL CBonDriverProxy::GetTsStream(BYTE **ppDst, DWORD *pdwSize, DWORD *pdwRemain)
{
	// Xg[|C^擾
	return (m_pTsTuner->GetStream(ppDst, pdwSize, pdwRemain))? TRUE : FALSE;
}

void CBonDriverProxy::PurgeTsStream(void)
{
	// ̃Xg[j
	m_pTsTuner->PurgeStream();
}

// IBonDriver2
LPCTSTR CBonDriverProxy::GetTunerName(void)
{
	static TCHAR szTunerName[1024];
	
	// `[iԂ(obt@pĂ邽ߑ̃CX^Xɂe)
	return (dynamic_cast<IHalDevice *>(m_pTsTuner)->GetDeviceName(szTunerName))? szTunerName : NULL;
}

const BOOL CBonDriverProxy::IsTunerOpening(void)
{
	// I[v̗LԂ
	return m_bIsOpen;
}

LPCTSTR CBonDriverProxy::EnumTuningSpace(const DWORD dwSpace)
{
	// `[jOԖԂ
	return (dwSpace < m_TuningSpace.size())? m_TuningSpace[dwSpace].strSpaceName.c_str() : NULL;
}

LPCTSTR CBonDriverProxy::EnumChannelName(const DWORD dwSpace, const DWORD dwChannel)
{
	// `lԂ
	if(dwSpace < m_TuningSpace.size()){
		return (dwChannel < m_TuningSpace[dwSpace].astrChannelName.size())? m_TuningSpace[dwSpace].astrChannelName[dwChannel].c_str() : NULL;
		}
	
	return NULL;
}

const BOOL CBonDriverProxy::SetChannel(const DWORD dwSpace, const DWORD dwChannel)
{
	// `lύX
	return (m_pTsTuner->SetChannel(dwSpace, dwChannel))? TRUE : FALSE;
}

const DWORD CBonDriverProxy::GetCurSpace(void)
{
	// ݂̃`[jOԂԂ
	return m_pTsTuner->GetCurSpace();
}

const DWORD CBonDriverProxy::GetCurChannel(void)
{
	// ݂̃`lԂ
	return m_pTsTuner->GetCurChannel();
}

// IBonDriver3
const DWORD CBonDriverProxy::GetTotalDeviceNum(void)
{
	// foCX̑Ԃ
	return dynamic_cast<IHalDevice *>(m_pTsTuner)->GetTotalDeviceNum();
}

const DWORD CBonDriverProxy::GetActiveDeviceNum(void)
{
	// gp̃foCXԂ
	return dynamic_cast<IHalDevice *>(m_pTsTuner)->GetActiveDeviceNum();
}

const BOOL CBonDriverProxy::SetLnbPower(const BOOL bEnable)
{
	// LNBdON/OFFݒ肷
	return m_pTsTuner->SetLnbPower((bEnable)? true : false);
}

CBonDriverProxy::CBonDriverProxy(LPCTSTR lpszClassName)
	: m_pTsTuner(::BON_SAFE_CREATE<IHalTsTuner *>(lpszClassName))
	, m_bIsOpen(FALSE)
{
	// `[iCX^XmF
	::BON_ASSERT(m_pTsTuner != NULL, TEXT("BonNX %s ̃CX^X𐶐ł܂"), lpszClassName);

	// RtBM[V
	::BON_ASSERT(LoadConfiguration(), TEXT("BonDriver̃RtBM[V[hs"));

	// `[jOԃ[h
	LoadTuningSpace();
}

CBonDriverProxy::~CBonDriverProxy()
{
	// RtBM[V
	::BON_ASSERT(SaveConfiguration(), TEXT("BonDriver̃RtBM[VZ[us"));

	// `[iCX^XJ
	BON_SAFE_RELEASE(m_pTsTuner);
}

void CBonDriverProxy::Release(void)
{
	// CX^XJ
	delete this;
}

const bool CBonDriverProxy::LoadConfiguration(void)
{
	bool bReturn = true;
	IConfigProperty *pProperty = NULL;
	IConfigStorage *pStorage = NULL;

	// vpeBǂݍ
	try{
		// vpeBNX擾
		TCHAR szPropertyClassName[BON_MAX_CLASS_NAME] = TEXT("");
		IConfigTarget *pConfigTarget = dynamic_cast<IConfigTarget *>(m_pTsTuner);
		if(!pConfigTarget)return true;
		pConfigTarget->GetPropertyClassName(szPropertyClassName);
						
		// vpeBCX^X
		pProperty = ::BON_SAFE_CREATE<IConfigProperty *>(szPropertyClassName);
		if(!pProperty)throw ::BON_EXPECTION(TEXT("vpeBCX^Xs")); 
		
		// INIXg[WCX^X
		pStorage = ::BON_SAFE_CREATE<IConfigStorage *>(TEXT("CIniCfgStorage"));
		if(!pStorage)throw ::BON_EXPECTION(TEXT("INIXg[WCX^Xs")); 

		// INIZNVI[v
		if(!pStorage->OpenStorage(TEXT("BonDriver_PT1")))throw ::BON_EXPECTION(TEXT("Xg[WI[vs")); 

		// vpeB[h
		if(!pProperty->LoadProperty(pStorage))throw ::BON_EXPECTION(TEXT("BonDriver_PT1vpeB[hs")); 

		// vpeBݒ
		if(!pConfigTarget->SetProperty(pProperty))throw ::BON_EXPECTION(TEXT("BonDriver_PT1vpeBݒ莸s")); 

		// INIXg[WN[Y
		pStorage->CloseStorage();
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		bReturn = false;
		}

	// CX^XJ
	BON_SAFE_RELEASE(pProperty);
	BON_SAFE_RELEASE(pStorage);

	return bReturn;
}

const bool CBonDriverProxy::SaveConfiguration(void)
{
	bool bReturn = true;
	IConfigProperty *pProperty = NULL;
	IConfigStorage *pStorage = NULL;

	// vpeBۑ
	try{
		// vpeBNX擾
		TCHAR szPropertyClassName[BON_MAX_CLASS_NAME] = TEXT("");
		IConfigTarget *pConfigTarget = dynamic_cast<IConfigTarget *>(m_pTsTuner);
		if(!pConfigTarget)return true;
		pConfigTarget->GetPropertyClassName(szPropertyClassName);
						
		// vpeBCX^X
		pProperty = ::BON_SAFE_CREATE<IConfigProperty *>(szPropertyClassName);
		if(!pProperty)throw ::BON_EXPECTION(TEXT("vpeBCX^Xs")); 
		
		// INIXg[WCX^X
		pStorage = ::BON_SAFE_CREATE<IConfigStorage *>(TEXT("CIniCfgStorage"));
		if(!pStorage)throw ::BON_EXPECTION(TEXT("INIXg[WCX^Xs")); 

		// INIZNVI[v
		if(!pStorage->OpenStorage(TEXT("BonDriver_PT1")))throw ::BON_EXPECTION(TEXT("Xg[WI[vs")); 

		// vpeB擾
		if(!pConfigTarget->GetProperty(pProperty))throw ::BON_EXPECTION(TEXT("BonDriver_PT1vpeB擾s")); 

		// vpeBZ[u
		if(!pProperty->SaveProperty(pStorage))throw ::BON_EXPECTION(TEXT("BonDriver_PT1vpeBZ[us")); 

		// INIXg[WN[Y
		pStorage->CloseStorage();
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		bReturn = false;
		}

	// CX^XJ
	BON_SAFE_RELEASE(pProperty);
	BON_SAFE_RELEASE(pStorage);

	return bReturn;
}

void CBonDriverProxy::LoadTuningSpace(void)
{
	// SẴ`l񋓂
	TCHAR szName[1024];

	DWORD dwSpace = 0UL;
	
	while(m_pTsTuner->EnumTuningSpace(dwSpace, szName)){
		// ԖZbg
		m_TuningSpace.resize(dwSpace + 1UL);
		m_TuningSpace[dwSpace].strSpaceName = szName;

		// `lZbg
		DWORD dwChannel = 0UL;
		
		while(m_pTsTuner->EnumChannelName(dwSpace, dwChannel++, szName)){
			m_TuningSpace[dwSpace].astrChannelName.push_back(std::wstring(szName));
			}

		dwSpace++;
		}
}
